@@ -400,6 +400,7 @@ G_LOCK_DEFINE (auth_cache);
 static GHashTable *auth_cache_basic;
 static GHashTable *auth_cache_proxy;
 static guint       cleanup_id;
+static guint       module_refcount = 0;
 
 typedef struct {
 	
@@ -2450,6 +2451,7 @@ do_create (GnomeVFSMethod	 *method,
 
 	hctx = handle->context;	
 
+put_start:
 	req = ne_request_create (hctx->session, "PUT", hctx->path);
 
 	/* Apache seems to not handle "If-None-Match: *" correctly 
@@ -2472,6 +2474,20 @@ do_create (GnomeVFSMethod	 *method,
 	ne_set_request_body_buffer (req, NULL, 0);
 	
 	res = ne_request_dispatch (req);
+
+	if (res == NE_REDIRECT) {
+		ne_request_destroy (req);
+		req = NULL;
+		
+		result = http_follow_redirect (hctx);
+		
+		if (result == GNOME_VFS_OK) {
+			goto put_start;
+		} else {
+			return result;
+		}
+	}
+
 	result 	= resolve_result (res, req);
 	ne_request_destroy (req);
 	
@@ -3331,20 +3347,24 @@ static GnomeVFSMethod http_method = {
 GnomeVFSMethod *
 vfs_module_init (const char *method_name, const char *args)
 {
-	proxy_init ();
-	/* ne_debug_init (stdout, 0xfffe); */
-	neon_session_pool_init ();
-	http_auth_cache_init ();
-	quick_allow_lookup_init ();
-	
+	if (module_refcount++ == 0) {
+		proxy_init ();
+		/* ne_debug_init (stdout, 0xfffe); */
+		neon_session_pool_init ();
+		http_auth_cache_init ();
+		quick_allow_lookup_init ();
+	}
+
 	return &http_method;
 }
 
 void
 vfs_module_shutdown (GnomeVFSMethod *method)
 {
-	quit_allow_lookup_destroy ();
-	http_auth_cache_shutdown ();
-	neon_session_pool_shutdown ();
-	proxy_shutdown ();
+	if (--module_refcount == 0) {
+		quit_allow_lookup_destroy ();
+		http_auth_cache_shutdown ();
+		neon_session_pool_shutdown ();
+		proxy_shutdown ();
+	}
 }
